// SolidCli implements a command line interface
// to the Solid game machinery.

#include "solid.hpp"
#include "solidint.hpp"
#include "solidcli.hpp"


#include <iostream>
#include <iomanip>


SolidCli::SolidCli( Solid* p ) : SolidInt( p )
{
    // nothing needed -- it's done in the base SolidInt class
}

void
SolidCli::Roll() const
{
    gp->Roll();

    int k;

    for ( k=0; k < 5; k++ )
    {
        std::cout << gp->Die( k ) << " ";
    }
    std::cout << "\n";

    for ( k=0; k < 5; k++ )
    {
        char c;
        c = 'a' + k;
        
        std::cout << c << " ";
    }
    std::cout << "\n";
}

void
SolidCli::Display() const
{
    int h, j, k;

    std::cout << "\n";

    gp->CommitTurn(); 

    for ( k=2; k < 13; k++ )
    {
        std::cout << std::setw(2) << k << " ";
        h = gp->Hits(k);
        for ( j=0; j < h; j++ ) {
            std::cout << "X";
            if ( j%5 == 4 )
                std::cout << "-";
        }
        std::cout << "\n";
    }

    if (gp->NumRejects())
    {
        std::cout << "\nrejects\n";
    }

    for ( k=1; k <= 6; k++ )
        if ( gp->RejectCounts(k) )
            std::cout << k << " " << gp->RejectCounts(k) << "\n";
    
    std::cout << "\nscore " << gp->Score() << "\n\n";
}

int
SolidCli::GetPairs() const
{
    int d1, d2, d3, d4;
    char c1, c2, c3, c4;

    int error;

    do {
        std::cout << ": ";
        std::cin >> c1 >> c2 >> c3 >> c4;

        d1 = c1 - 'a';
        d2 = c2 - 'a';
        d3 = c3 - 'a';
        d4 = c4 - 'a';
        
        if (c1 == 'q' || c1 == 'Q')
        {
            return 1;
        }

        if (d1 < 0 || d1 > 4 || d2 < 0 || d2 > 4 ||
            d3 < 0 || d3 > 4 || d4 < 0 || d4 > 4)
        {
            std::cout << "invalid letter\n";
            continue;
        }

        if (d1 == d2 || d1 == d3 || d1 == d4 ||
            d2 == d3 || d2 == d4 || d3 == d4)
        {
            std::cout << "duplicate letter\n";
            continue;
        }

        error = gp->SetGroup( d1,  1 );

        if (error)
        {
            std::cout << "internal error " << error << "\n";
            continue;
	}

        error = gp->SetGroup( d2, 1 );        

        if (error)
        {
            std::cout << "internal error " << error << "\n";
            continue;
	}

        error = gp->SetGroup( d3, 2 );

        if (error)
        {
            std::cout << "internal error " << error << "\n";
            continue;
	}

        error = gp->SetGroup( d4, 2 );        

        if (error)
        {
            std::cout << "internal error " << error << "\n";
            continue;
	}

        int j;

        for (j=0; j < 5; j++)
        {
            if ( j != d1 && j != d2 && j != d3 && j != d4 )
            {
                error = gp->SetGroup( j, 0 );

                if (error)
                {
                    std::cout << "invalid reject\n";
                }

                break;
            }
        }

    } while (error);

    return 0;
}
